@@ -1,10 +1,16 @@
 #include "cache.h"
 #include "string-list.h"
 
-void string_list_init(struct string_list *list, int strdup_strings)
+void string_list_init_nodup(struct string_list *list)
 {
-	memset(list, 0, sizeof(*list));
-	list->strdup_strings = strdup_strings;
+	struct string_list blank = STRING_LIST_INIT_NODUP;
+	memcpy(list, &blank, sizeof(*list));
+}
+
+void string_list_init_dup(struct string_list *list)
+{
+	struct string_list blank = STRING_LIST_INIT_DUP;
+	memcpy(list, &blank, sizeof(*list));
 }
 
 /* if there is no exact match, point to the index where the entry could be
@@ -155,7 +161,8 @@ static int item_is_not_empty(struct string_list_item *item, void *unused)
 	return *item->string != '\0';
 }
 
-void string_list_remove_empty_items(struct string_list *list, int free_util) {
+void string_list_remove_empty_items(struct string_list *list, int free_util)
+{
 	filter_string_list(list, free_util, item_is_not_empty, NULL);
 }
 
